#ifndef _AX_H_
#define _AX_H_

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <iostream.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/keysym.h>
#include <X11/extensions/Xdbe.h>
#include "axexception.h"
#include "axvector.h"

#define pointer(T)  typeof(T *)
#define array(T, N) typeof(T [N])

class axObject;
	class axEvent;
	class axWindow;
		class axWindowDBE;

typedef unsigned int uint;
typedef unsigned long ulong;
typedef void (axWindow::*EventCallBack)(XEvent&);


class axObject{
	private:
	bool trace_;

	public:
	axObject();
	void trace(bool val);
	void trace(const char *msg);
	void panic(const char *msg);
	void log(const char *msg);
};


class axWindow : public axObject{
	protected:
	char *name_;
	int width_, height_;
	int x_, y_;
	bool done_;
	char *displayname_;
	Display *display_;
	int screen_;
	Window window_;
	Drawable *drawable_; // for rerouting output in axWindowDBE
	GC gc_;
	axVector<axEvent> events_;

	public:
	axWindow(char *name = "axWindow", int width = 256, int height = 256);
	virtual ~axWindow();
	// member access
	bool done() { return done_; }
	void done(bool val) { done_ = val; }
	int x() { return x_; }
	void x(int val) { x_ = val; }
	int y() { return y_; }
	void y(int val) { y_ = val; }
	int width() { return width_; }
	void width(int val) { width_ = val; }
	int height() { return height_; }
	void height(int val) { height_ = val; }
	Display *display() { return display_; }
	Window window() { return window_; }
	int screen() { return screen_; }
	// axWindow specifics
	virtual void init();
	virtual void run();
	void addEvent(axEvent *event);
	void selectEvents(long mask);
	void ignoreEvents(long mask);
	virtual void checkEvents(); // XXX why virtual?
	virtual void eventLoop(); // XXX why virtual?
	// default event handlers
	virtual void onExpose(XEvent &event);
	virtual void onKeyPress(XEvent &event);
	virtual void onKeyRelease(XEvent &event);
	virtual void onButtonPress(XEvent &event);
	virtual void onButtonRelease(XEvent &event);
	// X wrappers
	void map();
	void unmap();
	void flush();
	void clearWindow();
	void getWindowAttributes(XWindowAttributes *wa);
	void setForeground(unsigned long pixelvalue);
	void setBackground(unsigned long pixelvalue);
	// X wrappers that operate on a drawable
	void drawPoint(int x, int y);
	void drawLine(int x1, int y1, int x2, int y2);
	void drawLines(XPoint *points, int npoints, int mode);
	void drawRectangle(int x, int y, uint width, uint height);
	void fillRectangle(int x, int y, uint width, uint height);
	void fillPolygon(XPoint *points, int npoints, int shape, int mode);
	void drawString(int x, int y, const char *str);
};


class axWindowDBE : public axWindow{
	private:
	Window back_;

	public:
	axWindowDBE(char *name, int width, int height);
	void init();
	void swapBuffers();
};


class axEvent : public axObject{
	private:
	friend axWindow;
	axWindow *parent_;
	int event_;
	EventCallBack callBack_;
	long mask_;

	public:
	axEvent(axWindow *parent, int event, EventCallBack callBack);
	virtual ~axEvent() { }
	void callBack(XEvent &event);
};

#endif _AX_H_
