#include "matrix.h"

void log(char *msg)
{
	cout << "*** " << msg << "\n";
}

void testVector()
{
	Vector v(1, 2, 3), w(2, 3, 4);
	cout << "v: " << v << "-v:" << -v;
	cout << "v + w: " << v + w;
	cout << "v * w: " << v * w;
	cout << "v / w: " << v / w;
	//cout << (v == v);
	cout << "vector magnitude: " << v.magnitude() << endl;
	cout << "normalized vector: " << v.normalize();
	cout << "normalized magnitude: " << v.normalize().magnitude() << endl;
	cout << "dotproduct: " << v.dotProduct(w) << endl;
	cout << "crossproduct: " << v.crossProduct(w);
	v(0) = 42 * v(1);
	cout << v;
	w = v;
	cout << w;
}

void testMatrix()
{
	Matrix m, n, o;
	Vector v;

	/* vector, matrix and io tests */
	m(0,0) = 2; m(1,0) = 3; m(2,0) = 4; m(3,0) = 5;
	n = m;
	cout << m; // << n;
	m.invert();
	cout << m;
	//cout << m.invert() * m << m * m.invert(); // identical ==> I
	m(0,0) = 1; m(0,1) = 2; m(0,2) = 3; m(0,3) = 0;
	m(1,0) = 1; m(1,1) = 2; m(1,2) = 3; m(1,3) = 0;
	m(2,0) = 1; m(2,1) = 2; m(2,2) = 3; m(2,3) = 0;
	m(3,0) = 0; m(3,1) = 0; m(3,2) = 0; m(3,3) = 1;
	n = Matrix::transpose(m);
	//cout << m << n;
	m = Matrix::transpose(m);
	//cout << m;
	v = Vector(1, 2, 3);
	//cout << v;
	v = m * m * v;
	cout << m * m << v;
	v = Vector(0, 0, 0);
	cout << v;
	m = Matrix::translateMatrix(Vector(1, 2, 3));
	cout << m;
	cout << m * v;
	v = Vector(1, 2, 3);
	m = Matrix::scaleMatrix(2);
	cout << m;
	cout << m * v;
	m = Matrix::rotateXMatrix(45);
	cout << m;
	n = Matrix::rotateYMatrix(45);
	cout << n;
	o = Matrix::rotateZMatrix(45);
	cout << o;
	cout << m * n * o;
	cout << Matrix::rotateXYZMatrix(Vector(45, 45, 45));
	v = Vector(0, 1, 0);
	cout << m * v;
	cout << n * v;
	cout << o * v;
	cout << m * n * o * v;
	cout << Matrix::rotateXYZMatrix(Vector(45, 45, 45)) * v;
	v = Vector(0, 1, 0);
	m = Matrix::translateMatrix(Vector(1, 0, 0));
	n = Matrix::scaleMatrix(2);
	cout << m * n << n * m << m * v << n * v << m * n * v << n * m * v;
	v = Vector(0, 1, 0);
	m.identity();
	m(3,0) = m(3,1) = m(3,2) = 1;
	cout << m << v << m * v;
}

void testDeterminant()
{
	Matrix m;
	cout << m << m.determinant() << endl << Matrix::invert(m);
	m(0,0) = 1; m(0,1) = 2; m(0,2) = 3; m(0,3) = 0;
	m(1,0) = 1; m(1,1) = 2; m(1,2) = 3; m(1,3) = 0;
	m(2,0) = 1; m(2,1) = 2; m(2,2) = 3; m(2,3) = 0;
	m(3,0) = 0; m(3,1) = 0; m(3,2) = 0; m(3,3) = 1;
	cout << m << m.determinant() << endl << Matrix::invert(m);
	m = Matrix::rotateXYZMatrix(Vector(45, 45, 45));
	cout << m << m.determinant() << endl << Matrix::invert(m);
	m = m * Matrix::scaleMatrix(sqrt(2));
	cout << m << m.determinant() << endl << Matrix::invert(m);
}


/* this does not work yet
Matrix v1(1, 3);
Matrix v2(3, 1);
Matrix v3(1, 1);
v3 = v1 * v2;
v1.print();
v2.print();
v3.print();
*/

int main()
{
	try{
		//testVector();
		//testMatrix();
		testDeterminant();
	}
	catch(MatrixException ex){
		cerr << ex;
	}
	return 0;
}

/*
Use operator overloading to provide a friend left-shift operator, operator<<. 

    #include <iostream.h>
    
    class Fred {
    public:
      friend ostream& operator<< (ostream& o, const Fred& fred);
      // ...
    private:
      int i_;    // Just for illustration
    };
    
    ostream& operator<< (ostream& o, const Fred& fred)
    {
      return o << fred.i_;
    }
    
    int main()
    {
      Fred f;
      cout << "My Fred object: " << f << "\n";
    } 

We use a non-member function (a friend in this case) since the Fred object is
the right-hand operand of the << operator. If the Fred object was supposed to
be on the left hand side of the << (that is, myFred << cout rather than cout
<< myFred), we could have used a member function named operator<<. 

Note that operator<< returns the stream. This is so the output operations can
be cascaded. 
*/
