#include "x3d.h"

Object::Object(Model *model)
{
	model_ = model;
	location_ = new Vector;
}

Object::~Object()
{
	delete[] vtcs_;
	delete[] view_;
	// FIXME delete planes_
}

int Object::readFromFile(char *filename)
{
	// TODO read from a stream
	FILE *fp = NULL;
	Plane *pp = NULL;
	Plane *tmp;
	int i, j, nplanes, nplanevtcs, vertex;
	double x, y, z;

	if((fp = fopen(filename, "rt")) == NULL){
		fprintf(stderr, "cannot open 3d-file '%s'\n", filename);
		return 0;
	}

	if(fscanf(fp, "%d %d", &nvertices_, &nplanes) != 2)
		return 0;

	vtcs_ = new Vector[nvertices_];
	view_ = new Vector[nvertices_];

	for(i = 0; i < nvertices_; i++){			// read vertex coords
		if(fscanf(fp, "%lf %lf %lf", &x, &y, &z) != 3)
			return 0;
		vtcs_[i].x() = x;
		vtcs_[i].y() = y;
		vtcs_[i].z() = z;
	}

	planes_ = NULL;

	for(i = 0; i < nplanes; i++){
			// this could be better, adds at the head of the list without an if
			//	p->SetNext((ants) ? ants : NULL);
			//	ants = p;
		if(!planes_){												// create first plane
			planes_ = new Plane(this);
			pp = planes_;
		}
		else{																// append successive planes
			tmp = new Plane(this);
			pp->next(tmp);
			pp = tmp;
		}

		// read nr of vertices in current plane
		if(fscanf(fp, "%d", &nplanevtcs) != 1)
			return 0;
		pp->nvertices(nplanevtcs);

		// read all vertex numbers for current plane
		for(j = 0; j < nplanevtcs; j++){
			if(fscanf(fp, "%d", &vertex) != 1)
				return 0;
			// correction for right-handed coord system:
			// vertices need to be ordered counter-clockwise to get the normals
			// right... Was: pp->vertex(j, vertex);
			pp->vertex(nplanevtcs - j - 1, vertex);
		}
	}

	fclose(fp);
	return 1;
}

void Object::makeCube(const Vector& location, int size)
{
	const int NPLANES = 6;
	nvertices_ = 8;
	vtcs_ = new Vector[nvertices_];
	view_ = new Vector[nvertices_];
	vtcs_[0] = Vector(-size, -size, -size);
	vtcs_[1] = Vector( size, -size, -size);
	vtcs_[2] = Vector( size,  size, -size);
	vtcs_[3] = Vector(-size,  size, -size);
	vtcs_[4] = Vector(-size, -size,  size);
	vtcs_[5] = Vector( size, -size,  size);
	vtcs_[6] = Vector( size,  size,  size);
	vtcs_[7] = Vector(-size,  size,  size);
	for(int i = 0; i < nvertices_; i++)
		vtcs_[i] = vtcs_[i] + location;
	planes_ = NULL;
	for(int i = 0; i < NPLANES; i++){
		Plane *pp = new Plane(this);
		if(planes_ != NULL)
			pp->next(planes_);
		planes_ = pp;
		pp->nvertices(4);
		switch(i){
			case 0:
				pp->vertex(3, 0); pp->vertex(2, 1); pp->vertex(1, 2); pp->vertex(0, 3);
				break;
			case 1:
				pp->vertex(3, 1); pp->vertex(2, 5); pp->vertex(1, 6); pp->vertex(0, 2);
				break;
			case 2:
				pp->vertex(3, 5); pp->vertex(2, 4); pp->vertex(1, 7); pp->vertex(0, 6);
				break;
			case 3:
				pp->vertex(3, 0); pp->vertex(2, 3); pp->vertex(1, 7); pp->vertex(0, 4);
				break;
			case 4:
				pp->vertex(3, 0); pp->vertex(2, 4); pp->vertex(1, 5); pp->vertex(0, 1);
				break;
			case 5:
				pp->vertex(3, 3); pp->vertex(2, 2); pp->vertex(1, 6); pp->vertex(0, 7);
		}
	}
}

void Object::draw(Matrix &rotation)
{
	rotate(rotation);
	trans3Dto2D();

	if(model_->hidden()){
		removeHidden();
		if(model_->color())
			setColors();
	}

	for(Plane *pp = planes_; pp; pp = pp->next())
		pp->draw(model_, view_, model_->color());

	// draw an x-axis | OLD-MODEL!
	/*
	Vector vx1, vx2;
	Vector x1(-300, 0, 0);
	Vector x2(300, 0, 0);
	double pointdist, viewdist;
	rotation.premultiply(x1);
	rotation.premultiply(x2);
	viewdist = model_->zviewplane() - model_->zprojpoint();
	pointdist = x1.z - model_->zprojpoint();
	vx1.x = (x1.x * viewdist) / pointdist;
	vx1.y = (x1.y * viewdist) / pointdist;
	pointdist = x2.z - model_->zprojpoint();
	vx2.x = (x2.x * viewdist) / pointdist;
	vx2.y = (x2.y * viewdist) / pointdist;
	vx1.x += 256;
	vx2.x += 256;
	vx1.y = 256 - vx1.y;
	vx2.y = 256 - vx2.y;
	model_->setForeground(0);
	model_->drawLine(vx1.x, vx1.y, vx2.x, vx2.y);
	*/
}		

void Object::rotate(Matrix &rotation)
{
	Vector v;

	for(int i = 0; i < nvertices_; i++)
		view_[i] = rotation * vtcs_[i];
}

void Object::trans3Dto2D()
{
	// see Hearn, pp. 445
	double h;														// homogeneous factor
	double zvp  = model_->zviewplane();	// z view point
	double zprp = model_->zprojpoint(); // z projection point
	double dp   = zprp - zvp;						// projection distance
	Vector v, w;
	Matrix ppt;													// perspective projection transformation
	ppt(2,2) = zvp / dp;
	ppt(2,3) = -zvp * (zprp / dp);
	ppt(3,2) = 1 / dp;
	ppt(3,3) = -zprp / dp;

	for(int i = 0; i < nvertices_; i++){
		v = view_[i];
		w = ppt * v;
		h = (v.z() - zprp) / dp;
		view_[i].x() = w.x() / h;
		view_[i].y() = w.y() / h;
	}
}

void Object::removeHidden()
{
	for(Plane *pp = planes_; pp; pp = pp->next())
		pp->removeHidden(view_);
}

void Object::makeAllVisible()
{
	for(Plane *pp = planes_; pp; pp = pp->next())
		pp->visible(1);
}

void Object::setColors()
{
	for(Plane *pp = planes_; pp; pp = pp->next())
		pp->setColor(*model_->light());
}
