#ifndef _X3D_H_
#define _X3D_H_

#include "ax/ax.h"
#include "matrix/matrix.h"

class Plane;
class Object;
class Model;

class Plane{ // XXX make a PlaneList (and this a generic List) with iterators?
	private:
	int nvertices_;
	int vertices_[4];
	int visible_;
	unsigned long color_;
	Vector normal_;
	Object *object_;
	Plane *next_;

	public:
	Plane(Object *object);
	Plane *next() { return next_; }
	void next(Plane *val) { next_ = val; }
	void nvertices(int val) { nvertices_ = val; }
	void vertex(int n, int val) { vertices_[n] = val; }
	void visible(int val) { visible_ = val; }
	void removeHidden(Vector *view);
	void setColor(Vector &light);
	void draw(Model *model, Vector *view, int coloring);
};

class Object{
	private:
	Vector *vtcs_; // constructing vertices
	Vector *view_; // 3dto2d transformated viewing vertices
	Vector *location_;
	Plane *planes_;
	int nvertices_;
	int color_;
	Model *model_;

	public:
	Object(Model *model);
	~Object();
	//int color() { return color_; }
	//void color(int val) { color_ = val; }
	Vector *location() { return location_; }
	void location(int x, int y, int z)
		{ location_->x() = x; location_->y() = y; location_->z() = z; }
	Model *model() { return model_; }
	int readFromFile(char *filename);
	void makeCube(const Vector& location, int size);
	void draw(Matrix &rotation);
	void rotate(Matrix &rotation);
	void trans3Dto2D();
	void removeHidden();
	void makeAllVisible();
	void setColors();
};

class Model : public axWindowDBE{
	// TODO create a Camera object for a viewpoint stuff
	// TODO create a full-blown Light object with colors, attenuation and stuff
	Object *object;
	Vector *light_;
	int zviewplane_;
	int zprojpoint_;
	int hidden_;
	int color_;
	int xangle, yangle, zangle;						// current rotation status
	int xstep, ystep, zstep;							// inc/dec control
	int mousectrl;												// control object with mouse
	int lastx, lasty;
	char *modelfile_;

	public:
	Model(char* modelfile = 0);
	Vector *light() { return light_; }
	int hidden() { return hidden_; }
	int color() { return color_; }
	int zviewplane() { return zviewplane_; }
	int zprojpoint() { return zprojpoint_; }
	void run();
	void onExpose(XEvent &ev);
	void onKeyPress(XEvent &ev);
	void onButtonPress(XEvent &ev);
	void onButtonRelease(XEvent &ev);
	void onMotionNotify(XEvent &ev);
};

#endif
